@@ -107,7 +107,7 @@ from .export import ExportManager
 from .extra import get_a, post_a, put_a, delete_a, patch_a, get_w, post_w, put_w, delete_w, patch_w
 from .geo import GeoResolver
 from .git import Git
-from .http import get_f, get, post, put, delete, HTTPResponse
+from .http import file_g, get_f, get, post, put, delete, HTTPResponse
 from .log import MemoryHandler, BaseFormatter, ThreadFormatter, DummyLogger, rotating_handler, smtp_handler,\
     in_signature
 from .meta import Ordered, Indexed
@@ -127,7 +127,7 @@ from .settings import DEBUG, USERNAME, PASSWORD
 from .smtp import message, message_base, message_netius, smtp_engine, multipart, plain,\
     html, header
 from .storage import StorageEngine, BaseEngine, FsEngine
-from .structures import OrderedDict, LazyDict, LazyValue, lazy_dict, lazy
+from .structures import OrderedDict, LazyDict, LazyValue, GeneratorFile, lazy_dict, lazy
 from .typesf import AbstractType, Type, File, Files, ImageFile, ImageFiles, image, images, Reference,\
     reference, References, references, Encrypted, encrypted, secure
 from .util import is_iterable, is_mobile, is_tablet, is_browser, is_bot, browser_info, email_parts, email_mime,\
@@ -139,10 +139,9 @@ from .util import is_iterable, is_mobile, is_tablet, is_browser, is_bot, browser
     dict_merge, cached, private, ensure, delayed, route, error_handler, exception_handler, before_request,\
     after_request, is_detached, sanitize, verify, verify_equal, verify_not_equal, execute, ctx_locale, ctx_request,\
     FileTuple, BaseThread, JSONEncoder
-from .validation import validate, validate_b, validate_e, safe, eq, gt, gte, lt, lte, not_null,\
-    not_empty, not_false, is_in, is_lower, is_simple, is_email, is_url, is_regex, field_eq, field_gt,\
-    field_gte, field_lt, field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate,\
-    all_different, no_self
+from .validation import validate, validate_b, validate_e, safe, eq, gt, gte, lt, lte, not_null, not_empty, not_false,\
+    is_in, is_upper, is_lower, is_simple, is_email, is_url, is_regex, field_eq, field_gt, field_gte, field_lt,\
+    field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate, all_different, no_self
 
 from .amqp import get_connection as get_amqp
 from .amqp import properties as properties_amqp
