@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -49,6 +49,7 @@ from . import http
 from . import legacy
 from . import observer
 from . import exceptions
+from . import structures
 
 class API(observer.Observable):
     """
@@ -85,10 +86,12 @@ class API(observer.Observable):
         redirect = None,
         timeout = None,
         callback = True,
+        extra = None,
         **kwargs
     ):
         headers = headers or dict()
-        params = params or kwargs
+        extra = extra or dict()
+        params = structures.OrderedDict(params or kwargs)
         auth_callback = self.auth_callback if callback else None
         self.build(
             "GET",
@@ -106,7 +109,8 @@ class API(observer.Observable):
             silent = silent,
             redirect = redirect,
             timeout = timeout,
-            auth_callback = auth_callback
+            auth_callback = auth_callback,
+            **extra
         )
 
     def post(
@@ -123,10 +127,12 @@ class API(observer.Observable):
         redirect = None,
         timeout = None,
         callback = True,
+        extra = None,
         **kwargs
     ):
         headers = headers or dict()
-        params = params or kwargs
+        extra = extra or dict()
+        params = structures.OrderedDict(params or kwargs)
         auth_callback = self.auth_callback if callback else None
         self.build(
             "POST",
@@ -152,7 +158,8 @@ class API(observer.Observable):
             silent = silent,
             redirect = redirect,
             timeout = timeout,
-            auth_callback = auth_callback
+            auth_callback = auth_callback,
+            **extra
         )
 
     def put(
@@ -169,10 +176,12 @@ class API(observer.Observable):
         redirect = None,
         timeout = None,
         callback = True,
+        extra = None,
         **kwargs
     ):
         headers = headers or dict()
-        params = params or kwargs
+        extra = extra or dict()
+        params = structures.OrderedDict(params or kwargs)
         auth_callback = self.auth_callback if callback else None
         self.build(
             "PUT",
@@ -198,7 +207,8 @@ class API(observer.Observable):
             silent = silent,
             redirect = redirect,
             timeout = timeout,
-            auth_callback = auth_callback
+            auth_callback = auth_callback,
+            **extra
         )
 
     def delete(
@@ -211,10 +221,12 @@ class API(observer.Observable):
         redirect = None,
         timeout = None,
         callback = True,
+        extra = None,
         **kwargs
     ):
         headers = headers or dict()
-        params = params or kwargs
+        extra = extra or dict()
+        params = structures.OrderedDict(params or kwargs)
         auth_callback = self.auth_callback if callback else None
         self.build(
             "DELETE",
@@ -232,7 +244,8 @@ class API(observer.Observable):
             silent = silent,
             redirect = redirect,
             timeout = timeout,
-            auth_callback = auth_callback
+            auth_callback = auth_callback,
+            **extra
         )
 
     def patch(
@@ -249,10 +262,12 @@ class API(observer.Observable):
         redirect = None,
         timeout = None,
         callback = True,
+        extra = None,
         **kwargs
     ):
         headers = headers or dict()
-        params = params or kwargs
+        extra = extra or dict()
+        params = structures.OrderedDict(params or kwargs)
         auth_callback = self.auth_callback if callback else None
         self.build(
             "PATCH",
@@ -278,7 +293,8 @@ class API(observer.Observable):
             silent = silent,
             redirect = redirect,
             timeout = timeout,
-            auth_callback = auth_callback
+            auth_callback = auth_callback,
+            **extra
         )
 
     def request(self, method, *args, **kwargs):
