@@ -40,6 +40,7 @@ __license__ = "Apache License, Version 2.0"
 import sys
 import threading
 
+from . import util
 from . import config
 from . import legacy
 from . import exceptions
@@ -58,12 +59,27 @@ class AsyncManager(object):
     def stop(self):
         pass
 
-    def add(self, method, args, kwargs, request = None, mid = None):
+    def restart(self):
+        if self.running: self.stop()
+        self.start()
+
+    def add(self, method, args = [], kwargs = {}, request = None, mid = None):
         pass
 
-class SimpleManager(AsyncManager):
+    @property
+    def running(self):
+        return False
 
-    def add(self, method, args, kwargs, request = None, mid = None):
+class SimpleManager(AsyncManager):
+    """
+    Simple (and resource intensive) async manager that
+    creates a new thread for every async execution context.
+
+    Should only be used for testing and debugging purposes
+    and be avoided in production environments.
+    """
+
+    def add(self, method, args = [], kwargs = {}, request = None, mid = None):
         if request: kwargs["request"] = request
         if mid: kwargs["mid"] = mid
         thread = threading.Thread(
@@ -73,23 +89,41 @@ class SimpleManager(AsyncManager):
         )
         thread.start()
 
+    @property
+    def running(self):
+        return True
+
 class QueueManager(AsyncManager):
+    """
+    Queue based async manager that uses a single consumer
+    thread to consume work (method calls) introduced into
+    a simple FIFO queue.
+    """
 
     def __init__(self, owner):
         AsyncManager.__init__(self, owner)
         self.queue = []
-        self.condition = threading.Condition()
+        self.condition = None
+        self._running = False
 
     def start(self):
+        util.verify(not self._running)
+        self._running = True
         self.thread = threading.Thread(target = self.handler)
-        self.running = True
         self.thread.daemon = True
+        self.condition = threading.Condition()
         self.thread.start()
 
     def stop(self):
-        self.running = False
+        util.verify(self._running)
+        self._running = False
+        self.condition.acquire()
+        try: self.condition.notify()
+        finally: self.condition.release()
+        self.thread.join()
 
-    def add(self, method, args, kwargs, request = None, mid = None):
+    def add(self, method, args = [], kwargs = {}, request = None, mid = None):
+        util.verify(self.running)
         if request: kwargs["request"] = request
         if mid: kwargs["mid"] = mid
         item = (method, args, kwargs)
@@ -101,11 +135,32 @@ class QueueManager(AsyncManager):
             self.condition.release()
 
     def handler(self):
+        # iterates continuously while the running flag is set,
+        # this flag should be changed by a different thread
+        # that aims at stopping this one (and maybe join it)
         while self.running:
+            # acquires the condition and waits until the condition
+            # is set and the queue has some "real" items
             self.condition.acquire()
-            while not self.queue: self.condition.wait()
-            try: item = self.queue.pop(0)
-            finally: self.condition.release()
+            while not self.queue and self.running:
+                self.condition.wait()
+
+            # in case the thread is meant to be stopped, stops the
+            # current loop immediately
+            if not self.running: break
+
+            try:
+                # retrieves the latest item in the queue that is going
+                # to be processed as soon as possible
+                item = self.queue.pop(0)
+            finally:
+                # releases the condition lock as an item from the queue
+                # has been correctly scheduled for processing
+                self.condition.release()
+
+            # unpacks the current item (work unit) into the method and
+            # the arguments and runs the call handling a possible exception
+            # gracefully (prints exception to the logger)
             method, args, kwargs = item
             try: method(*args, **kwargs)
             except BaseException as exception:
@@ -114,6 +169,10 @@ class QueueManager(AsyncManager):
                     message = "Problem handling async item: %s"
                 )
 
+    @property
+    def running(self):
+        return self._running
+
 class AwaitWrapper(object):
     pass
 
