@@ -60,7 +60,7 @@ IMPORT_NAMES = ("$import", "$include", "$IMPORT", "$INCLUDE")
 name that references a list of include files to be loaded """
 
 CASTS = {
-    bool : lambda v: v if isinstance(v, bool) else v == "1",
+    bool : lambda v: v if isinstance(v, bool) else v in ("1", "true", "True"),
     list : lambda v: v if isinstance(v, list) else v.split(";"),
     tuple : lambda v: v if isinstance(v, tuple) else tuple(v.split(";"))
 }
@@ -92,10 +92,8 @@ HOMES = []
 to be the home on in terms of configuration, this value should
 be set on the initial loading of the ".home" file """
 
-__builtins__ = __builtins__ if isinstance(__builtins__, dict) else\
-    __builtins__.__dict__
-""" The global builtins reference created by the proper redefinition
-of the variable if that's required by python implementation """
+if not isinstance(__builtins__, dict):
+    __builtins__ = __builtins__.__dict__
 
 def conf(name, default = None, cast = None):
     """
@@ -120,9 +118,7 @@ def conf(name, default = None, cast = None):
     name or the default value if no value was found.
     """
 
-    is_string = type(cast) in legacy.STRINGS
-    if is_string: cast = __builtins__.get(cast, None)
-    cast = CASTS.get(cast, cast)
+    cast = _cast_r(cast)
     value = CONFIGS.get(name, default)
     if cast and not value == None: value = cast(value)
     return value
@@ -258,6 +254,12 @@ def get_homes(
 
     return HOMES
 
+def _cast_r(cast):
+    is_string = type(cast) in legacy.STRINGS
+    if is_string: cast = __builtins__.get(cast, None)
+    if not cast: return None
+    return CASTS.get(cast, cast)
+
 def _load_includes(base_path, config, encoding = "utf-8"):
     includes = ()
 
