@@ -390,7 +390,7 @@ class Request(object):
 
     def set_header(self, name, value, normalize = False):
         if normalize: name = name.title()
-        self.out_headers[name] = str(value)
+        self.out_headers[name] = legacy.ascii(value)
 
     def set_headers(self, headers):
         if not headers: return
@@ -405,10 +405,44 @@ class Request(object):
         if insensitive: name = name.title()
         return name in self.out_headers
 
-    def get_address(self, resolve = True):
+    def get_address(self, resolve = True, cleanup = True):
         """
         Retrieves the client (network) address associated with the
-        correct request/connection.
+        current request/connection.
+
+        In case the resolve flag is set the resolution takes a more
+        complex approach to avoid the common problems when used proxy
+        based connections (indirection in connection).
+
+        If cleanup mode is used it means that IPv4 addresses encapsulated
+        in IPv6 addresses will have the prefix removed (eg: ::ffff:127.0.0.01
+        will become just 127.0.0.1).
+
+        :type resolve: bool
+        :param resolve: If the complex resolution process for the
+        network address should take place.
+        :type cleanup: bool
+        :param cleanup: If the final value should have the possible
+        IPv6 to IPv4 address prefix removed.
+        :rtype: String
+        :return: The resolved client (network) address.
+        """
+
+        if resolve:
+            address = self.get_header("X-Forwarded-For", self.address)
+            address = self.get_header("X-Client-IP", address)
+            address = self.get_header("X-Real-IP", address)
+            address = address.split(",", 1)[0].strip() if address else address
+        else:
+            address = self.address
+        if cleanup and address and address.startswith("::ffff:"):
+            address = address[7:]
+        return address
+
+    def get_host(self, resolve = True):
+        """
+        Retrieves the hostname (as a string) associated with the
+        current request.
 
         In case the resolve flag is set the resolution takes a more
         complex approach to avoid the common problems when used proxy
@@ -416,17 +450,24 @@ class Request(object):
 
         :type resolve: bool
         :param resolve: If the complex resolution process for the
-        network address should take place.
+        hostname should take place.
         :rtype: String
-        :return: The resolved client (network) address.
+        :return: The resolved hostname string value.
         """
 
-        if not resolve: return self.address
-        address = self.get_header("X-Forwarded-For", self.address)
-        address = self.get_header("X-Client-IP", address)
-        address = self.get_header("X-Real-IP", address)
-        address = address.split(",", 1)[0].strip()
-        return address
+        server_name = self.environ.get("SERVER_NAME", "localhost")
+        server_port = int(self.environ.get("SERVER_PORT", "80"))
+        server_host = "%s:%d" % (server_name, server_port)
+        host = self.get_header("Host", server_host)
+        if resolve: host = self.get_header("X-Forwarded-Host", host)
+        return host
+
+    def get_url(self, resolve = True):
+        host = self.get_host(resolve = resolve)
+        if not host: return
+        if not self.scheme: return
+        query_s = "?%s" % self.query if self.query else ""
+        return "%s://%s%s%s" % (self.scheme, host, self.path, query_s)
 
     def resolve_params(self):
         self.params = self._resolve_p(self.params)
@@ -532,6 +573,10 @@ class Request(object):
         value = value or self.locale
         return value.replace("-", "_").lower()
 
+    def locale_l(self, value = None):
+        value = value or self.locale
+        return value.split("_", 1)[0]
+
     def load_locale(self, available, fallback = "en_us", ensure = True):
         # tries to gather the best locale value using the currently
         # available strategies and in case the retrieved local is part
@@ -540,9 +585,11 @@ class Request(object):
         # locale in case the ensure flag is set)
         locale = self.get_locale(fallback = fallback)
         locale = self.locale_b(locale)
-        if locale in available: self.set_locale(locale)
-        elif ensure and available: self.set_locale(available[0])
-        else: self.set_locale(fallback)
+        language = self.locale_l(locale)
+        if locale in available: return self.set_locale(locale)
+        if language in available: return self.set_locale(locale)
+        if ensure and available: return self.set_locale(available[0])
+        return self.set_locale(fallback)
 
     def get_locale(self, fallback = "en_us"):
         # tries to retrieve the locale value from the provided URL
@@ -575,7 +622,7 @@ class Request(object):
         # sets both the base locale value but also the language attribute
         # that may be used to determine the base (less specific) language value
         self.locale = locale
-        self.language = locale.split("_", 1)[0]
+        self.language = self.locale_l(locale)
 
     def get_langs(self):
         # gathers the value of the accept language header and in case
