@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -455,6 +455,36 @@ class ModelTest(unittest.TestCase):
         self.assertEqual(person.father.car.is_resolved(), True)
         self.assertEqual(person.father.car.name, "CarFather")
 
+    def test_unresolvable(self):
+        person = mock.Person()
+        person.name = "Name"
+        person.save()
+
+        car = mock.Car()
+        car.name = "Car"
+        car.save()
+
+        person = mock.Person.get(identifier = 1)
+        person.car = car
+        person.save()
+
+        self.assertEqual(isinstance(person.car, appier.Reference), False)
+
+        person = mock.Person.get(identifier = 1)
+
+        self.assertEqual(isinstance(person.car, appier.Reference), True)
+        self.assertEqual(person.car.is_resolvable(), True)
+        self.assertEqual(person.car == None, False)
+        self.assertEqual(person.car.name, "Car")
+
+        car.delete()
+
+        person = mock.Person.get(identifier = 1)
+
+        self.assertEqual(isinstance(person.car, appier.Reference), True)
+        self.assertEqual(person.car.is_resolvable(), False)
+        self.assertEqual(person.car == None, False)
+
     def test_exists(self):
         person = mock.Person()
         person.name = "Name"
