@@ -292,6 +292,12 @@ class UtilTest(unittest.TestCase):
         self.assertEqual(name, appier.legacy.u("你好世界"))
         self.assertEqual(email, appier.legacy.u("hello@hive.pt"))
 
+        name, email = appier.email_parts(appier.legacy.u(" joamag@hive.pt "))
+        self.assertEqual(type(name), appier.legacy.UNICODE)
+        self.assertEqual(type(email), appier.legacy.UNICODE)
+        self.assertEqual(name, appier.legacy.u("joamag@hive.pt"))
+        self.assertEqual(email, appier.legacy.u("joamag@hive.pt"))
+
     def test_email_mime(self):
         result = appier.email_mime("João Magalhães <joamag@hive.pt>")
         self.assertEqual(type(result), str)
@@ -301,6 +307,45 @@ class UtilTest(unittest.TestCase):
         self.assertEqual(type(result), appier.legacy.UNICODE)
         self.assertEqual(result, appier.legacy.u("=?utf-8?q?Jo=C3=A3o_Magalh=C3=A3es?= <joamag@hive.pt>"))
 
+        result = appier.email_mime(appier.legacy.u(" joamag@hive.pt "))
+        self.assertEqual(type(result), appier.legacy.UNICODE)
+        self.assertEqual(result, appier.legacy.u("joamag@hive.pt <joamag@hive.pt>"))
+
+        result = appier.email_mime([
+            appier.legacy.u("João Magalhães <joamag@hive.pt>"),
+            appier.legacy.u(" joamag@hive.pt "),
+            None
+        ])
+        self.assertEqual(type(result), list)
+        self.assertEqual(result, [
+            appier.legacy.u("=?utf-8?q?Jo=C3=A3o_Magalh=C3=A3es?= <joamag@hive.pt>"),
+            appier.legacy.u("joamag@hive.pt <joamag@hive.pt>")
+        ])
+
+    def test_email_base(self):
+        result = appier.email_base("João Magalhães <joamag@hive.pt>")
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "joamag@hive.pt")
+
+        result = appier.email_base(appier.legacy.u("João Magalhães <joamag@hive.pt>"))
+        self.assertEqual(type(result), appier.legacy.UNICODE)
+        self.assertEqual(result, appier.legacy.u("joamag@hive.pt"))
+
+        result = appier.email_base(appier.legacy.u(" joamag@hive.pt "))
+        self.assertEqual(type(result), appier.legacy.UNICODE)
+        self.assertEqual(result, appier.legacy.u("joamag@hive.pt"))
+
+        result = appier.email_base([
+            appier.legacy.u("joamag@hive.pt"),
+            appier.legacy.u("joamag@hive.pt"),
+            None
+        ])
+        self.assertEqual(type(result), list)
+        self.assertEqual(result, [
+            appier.legacy.u("joamag@hive.pt"),
+            appier.legacy.u("joamag@hive.pt")
+        ])
+
     def test_date_to_timestamp(self):
         result = appier.date_to_timestamp("29/06/1984")
         self.assertEqual(type(result), int)
@@ -413,6 +458,30 @@ class UtilTest(unittest.TestCase):
         self.assertEqual(type(result), str)
         self.assertEqual(result, "Hello World Hello World")
 
+        result = appier.underscore_to_readable("hello_world_")
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "Hello world")
+
+        result = appier.underscore_to_readable("hello_world_", capitalize = True)
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "Hello World")
+
+        result = appier.underscore_to_readable("__hello_world__")
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "Hello world")
+
+        result = appier.underscore_to_readable("__hello_world__", capitalize = True)
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "Hello World")
+
+        result = appier.underscore_to_readable("__hello___world__")
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "Hello world")
+
+        result = appier.underscore_to_readable("__hello___world__", capitalize = True)
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "Hello World")
+
     def test_is_content_type(self):
         result = appier.is_content_type("text/plain", "text/plain")
         self.assertEqual(result, True)
@@ -715,3 +784,20 @@ class FileTupleTest(unittest.TestCase):
         self.assertEqual(file.name, "hello")
         self.assertEqual(file.mime, "text/plain")
         self.assertEqual(file.data, b"hello world")
+
+    def test_interface(self):
+        file = appier.FileTuple.from_data(
+            b"hello world",
+            name = "hello",
+            mime = "text/plain"
+        )
+
+        self.assertEqual(file.read(), b"hello world")
+        self.assertEqual(file.tell(), 11)
+
+        file.seek(0)
+
+        self.assertEqual(file.tell(), 0)
+
+        self.assertEqual(file.read(5), b"hello")
+        self.assertEqual(file.tell(), 5)
