@@ -299,7 +299,7 @@ def browser_info(user_agent):
 
     return info if info else None
 
-def email_parts(base, encoding = None):
+def email_parts(base, strip = True):
     """
     Unpacks the complete set of parts (name and email) from the
     provided generalized email string. The provided string may
@@ -312,6 +312,9 @@ def email_parts(base, encoding = None):
     :type base: String/List
     :param base: The base value that is going to be parsed as an
     email string or a sequence of such values.
+    :type strip: bool
+    :param strip: If the provided base value should be stripped
+    of any extra space characters before processing.
     :rtype: Tuple/List
     :return: The resulting parsed tuple/tuples for the provided
     email strings, these tuples contain name and emails for each
@@ -320,7 +323,10 @@ def email_parts(base, encoding = None):
 
     base_t = type(base)
     if base_t in SEQUENCE_TYPES:
-        return [email_parts(base) for base in base]
+        return [email_parts(base, strip = strip) for base in base]
+
+    if not base: return (None, None)
+    if strip: base = base.strip()
 
     match = defines.EMAIL_REGEX.match(base)
     if not match: return (None, None)
@@ -335,23 +341,26 @@ def email_mime(base, encoding = "utf-8"):
 
     base_t = type(base)
     if base_t in SEQUENCE_TYPES:
-        return [email_mime(item, encoding = encoding) for item in base]
+        return [value for value in (email_mime(item, encoding = encoding) for item in base) if value]
 
     name, email = email_parts(base)
+    if not name or not email: return None
+
     name = smtp.header(name, encoding = encoding)
+
     return "%s <%s>" % (name, email)
 
 def email_name(base):
     base_t = type(base)
     if base_t in SEQUENCE_TYPES:
-        return [email_base(base) for base in base]
+        return [value for value in (email_base(base) for base in base) if value]
     name, _email = email_parts(base)
     return name
 
 def email_base(base):
     base_t = type(base)
     if base_t in SEQUENCE_TYPES:
-        return [email_base(base) for base in base]
+        return [value for value in (email_base(base) for base in base) if value]
     _name, email = email_parts(base)
     return email
 
@@ -386,19 +395,23 @@ def ensure_pip(name, package = None, delayed = False):
         install_pip_s(package, delayed = delayed)
 
 def install_pip(package, delayed = False, user = False):
-    import pip
+    import pip #@UnusedImport
+    try: import pip._internal
+    except: pip._internal = None
     args = ["install", package]
+    if hasattr(pip._internal, "main"): pip_main = pip._internal.main
+    else: pip_main = pip.main #@UndefinedVariable
     if user: args.insert(1, "--user")
     if delayed:
         import multiprocessing
         process = multiprocessing.Process(
-            target = pip.main,
+            target = pip_main,
             args = (args,)
         )
         process.start()
         result = 0
     else:
-        result = pip.main(args)
+        result = pip_main(args)
     if result == 0: return
     raise exceptions.OperationalError(message = "pip error")
 
@@ -861,6 +874,7 @@ def underscore_to_readable(underscore, capitalize = False):
     """
 
     parts = underscore.split("_")
+    parts = [part for part in parts if part]
     if capitalize: parts = [part[0].upper() + part[1:] for part in parts]
     else: parts[0] = parts[0][0].upper() + parts[0][1:]
     return " ".join(parts)
@@ -1403,19 +1417,21 @@ def check_tokens(self, tokens, tokens_m = None, request = None):
         ): return False
     return True
 
-def ensure_login(self, token = None, request = None):
+def ensure_login(self, token = None, context = None, request = None):
     request = request or (self.request if self else None)
     is_auth = "username" in request.session
     if not is_auth: raise exceptions.AppierException(
         message = "User not authenticated",
         code = 403,
-        token = token
+        token = token,
+        context = context
     )
     if check_token(self, token, request = request): return
     raise exceptions.AppierException(
         message = "Not enough permissions",
         code = 403,
-        token = token
+        token = token,
+        context = context
     )
 
 def get_tokens_m(self, request = None, set = True):
@@ -1526,7 +1542,7 @@ def dict_merge(first, second, override = True, recursive = False):
             # it represents a dictionary (smart merge) or if instead
             # it's of any other type (no smart merge possible)
             other = second[key]
-            if isinstance(other, dict):
+            if isinstance(value, dict) and isinstance(other, dict):
                 final[key] = dict_merge(
                     value,
                     other,
@@ -1595,7 +1611,7 @@ def private(function):
 
     return _private
 
-def ensure(token = None):
+def ensure(token = None, context = None):
 
     def decorator(function):
 
@@ -1606,6 +1622,7 @@ def ensure(token = None):
             if ensure: ensure_login(
                 self,
                 token = token,
+                context = context,
                 request = request
             )
             sanitize(function, kwargs)
@@ -1624,7 +1641,7 @@ def delayed(function):
 
     return _delayed
 
-def route(url, method = "GET", asynchronous = False, json = False):
+def route(url, method = "GET", asynchronous = False, json = False, opts = None):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
@@ -1633,13 +1650,14 @@ def route(url, method = "GET", asynchronous = False, json = False):
             url,
             function,
             asynchronous = asynchronous,
-            json = json
+            json = json,
+            opts = opts
         )
         return function
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        route = (url, method, asynchronous, json)
+        route = (url, method, asynchronous, json, opts)
         if not hasattr(function, "_routes"): function._routes = []
         function._routes.append(route)
         function.creation_counter = CREATION_COUNTER
@@ -1647,16 +1665,21 @@ def route(url, method = "GET", asynchronous = False, json = False):
 
     return decorator
 
-def error_handler(code, scope = None, json = False):
+def error_handler(code, scope = None, json = False, opts = None):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
-        else: common.base().App.add_error(code, function, json)
+        else: common.base().App.add_error(
+            code,
+            function,
+            json = json,
+            opts = opts
+        )
         return function
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        error = (code, scope, json)
+        error = (code, scope, json, opts)
         if not hasattr(function, "_errors"): function._errors = []
         function._errors.append(error)
         function.creation_counter = CREATION_COUNTER
@@ -1665,16 +1688,21 @@ def error_handler(code, scope = None, json = False):
 
     return decorator
 
-def exception_handler(exception, scope = None, json = False):
+def exception_handler(exception, scope = None, json = False, opts = None):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
-        else: common.base().App.add_exception(exception, function, json)
+        else: common.base().App.add_exception(
+            exception,
+            function,
+            json = json,
+            opts = opts
+        )
         return function
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        _exception = (exception, scope, json)
+        _exception = (exception, scope, json, opts)
         if not hasattr(function, "_exceptions"): function._exceptions = []
         function._exceptions.append(_exception)
         function.creation_counter = CREATION_COUNTER
@@ -1683,16 +1711,16 @@ def exception_handler(exception, scope = None, json = False):
 
     return decorator
 
-def before_request(scope = "all"):
+def before_request(scope = "all", opts = None):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
-        else: common.base().App.add_custom("before_request", function)
+        else: common.base().App.add_custom("before_request", function, opts = opts)
         return function
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        _custom = ("before_request",)
+        _custom = ("before_request", opts)
         if not hasattr(function, "_customs"): function._customs = []
         function._customs.append(_custom)
         function.creation_counter = CREATION_COUNTER
@@ -1701,16 +1729,16 @@ def before_request(scope = "all"):
 
     return decorator
 
-def after_request(scope = "all"):
+def after_request(scope = "all", opts = None):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
-        else: common.base().App.add_custom("after_request", function)
+        else: common.base().App.add_custom("after_request", function, opts = opts)
         return function
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        _custom = ("after_request",)
+        _custom = ("after_request", opts)
         if not hasattr(function, "_customs"): function._customs = []
         function._customs.append(_custom)
         function.creation_counter = CREATION_COUNTER
@@ -1834,9 +1862,15 @@ class FileTuple(tuple):
     in the context of the appier infra-structure.
 
     This class shares many of the signature with the
-    typical python class interface.
+    typical python file interface, allowing most of
+    the operation to be performed (eg: read, seek,
+    tell, etc.).
     """
 
+    def __init__(self, *args, **kwargs):
+        tuple.__init__(*args, **kwargs)
+        self._position = 0
+
     @classmethod
     def from_data(cls, data, name = None, mime = None):
         file_tuple = cls((name, mime, data))
@@ -1863,8 +1897,23 @@ class FileTuple(tuple):
         return None
 
     def read(self, count = None):
-        contents = self[2]
-        return contents
+        data, data_l = self[2], len(self[2])
+        if not count and self._position == 0: data, offset = data, data_l
+        elif not count: data, offset = data[self._position:], data_l - self._position
+        else: data, offset = data[self._position:self._position + count], count
+        self._position += offset
+        return data
+
+    def seek(self, offset, whence = os.SEEK_SET):
+        if whence == os.SEEK_SET:
+            self._position = offset
+        if whence == os.SEEK_CUR:
+            self._position += offset
+        if whence == os.SEEK_END:
+            self._position = len(self[2]) + offset
+
+    def tell(self):
+        return self._position
 
     def save(self, path):
         contents = self[2]
